--[[
Copyright (C) GtX (Andy), 2022

Author: GtX | Andy
Date: 09.10.2022
Revision: FS22-01

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No modifications may be made to this script, including conversion to other game versions without written permission from GtX | Andy
Copying or removing any part of this code for external use without written permission from GtX | Andy is prohibited.

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
Ohne schriftliche Genehmigung von GtX | Andy dürfen keine Änderungen an diesem Skript vorgenommen werden, einschließlich der Konvertierung in andere Spielversionen
Das Kopieren oder Entfernen irgendeines Teils dieses Codes zur externen Verwendung ohne schriftliche Genehmigung von GtX | Andy ist verboten.
]]


PlaceableHusbandryMoveAnimals = {}

PlaceableHusbandryMoveAnimals.MOD_NAME = g_currentModName
PlaceableHusbandryMoveAnimals.SPEC_ENTRY_NAME = string.format("spec_%s.husbandryMoveAnimals", g_currentModName)

function PlaceableHusbandryMoveAnimals.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(PlaceableHusbandryAnimals, specializations)
end

function PlaceableHusbandryMoveAnimals.registerEventListeners(placeableType)
    SpecializationUtil.registerEventListener(placeableType, "onPostLoad", PlaceableHusbandryMoveAnimals)
end

function PlaceableHusbandryMoveAnimals:onPostLoad(savegame)
    local husbandryAnimalsSpec = self.spec_husbandryAnimals
    local animalLoadingTrigger = husbandryAnimalsSpec.animalLoadingTrigger

    -- Replace activatable here to add new input binding without making unnecessary global changes to the 'AnimalLoadingTriggerActivatable' as it may be used by buildings not requiring this new feature.
    if animalLoadingTrigger ~= nil and animalLoadingTrigger.activatable ~= nil then
        if not animalLoadingTrigger.isDealer and animalLoadingTrigger.husbandry == self then
            g_currentMission.activatableObjectsSystem:removeActivatable(animalLoadingTrigger.activatable)

            animalLoadingTrigger.activatable = AnimalLoadingTriggerMoveActivatable.new(animalLoadingTrigger)
            self[PlaceableHusbandryMoveAnimals.SPEC_ENTRY_NAME].activatableAdded = true
        end
    end
end


AnimalLoadingTriggerMoveActivatable = {}
AnimalLoadingTriggerMoveActivatable.MOD_NAME = g_currentModName

local AnimalLoadingTriggerMoveActivatable_mt = Class(AnimalLoadingTriggerMoveActivatable, AnimalLoadingTriggerActivatable)

function AnimalLoadingTriggerMoveActivatable.new(owningTrigger)
    local self = setmetatable({}, AnimalLoadingTriggerMoveActivatable_mt)

    self.owner = owningTrigger
    self.activateText = g_i18n:getText("animals_openAnimalScreen", owningTrigger.customEnvironment)

    self.activateActionEventId = nil
    self.moveAnimalsActionEventId = nil

    return self
end

function AnimalLoadingTriggerMoveActivatable:registerCustomInput(inputContext)
    local _, actionEventId = g_inputBinding:registerActionEvent(InputAction.ACTIVATE_OBJECT, self, self.actionEventActivate, false, true, false, true)

    g_inputBinding:setActionEventText(actionEventId, self.activateText)
    g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_HIGH)
    g_inputBinding:setActionEventTextVisibility(actionEventId, true)

    self.activateActionEventId = actionEventId

    if inputContext == Player.INPUT_CONTEXT_NAME then
        _, actionEventId = g_inputBinding:registerActionEvent(InputAction.MOVE_HUSBANDRY_ANIMALS, self, self.actionEventMoveAnimals, false, true, false, true)

        g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_VERY_HIGH)
        g_inputBinding:setActionEventTextVisibility(actionEventId, true)

        self.moveAnimalsActionEventId = actionEventId
    end
end

function AnimalLoadingTriggerMoveActivatable:removeCustomInput(inputContext)
    g_inputBinding:removeActionEventsByTarget(self)

    self.activateActionEventId = nil
    self.moveAnimalsActionEventId = nil
end

function AnimalLoadingTriggerMoveActivatable:actionEventActivate(actionName, inputValue, callbackState, isAnalog)
    self:run()
end

function AnimalLoadingTriggerMoveActivatable:actionEventMoveAnimals(actionName, inputValue, callbackState, isAnalog)
    if self.owner ~= nil and self.owner.husbandry ~= nil then
        local sourceHusbandry = self.owner.husbandry

        local animalTypeIndex = sourceHusbandry:getAnimalTypeIndex()
        local farmId = sourceHusbandry:getOwnerFarmId()

        local husbandries = {}
        local lastTargetHusbandryIndex = 1

        for _, husbandry in ipairs(g_currentMission.husbandrySystem.placeables) do
            if husbandry ~= sourceHusbandry and husbandry:getOwnerFarmId() == farmId then
                if animalTypeIndex == husbandry:getAnimalTypeIndex() then
                    table.insert(husbandries, husbandry)

                    if husbandry == self.lastTargetHusbandry then
                        lastTargetHusbandryIndex = #husbandries
                    end
                end
            end
        end

        if #husbandries > 0 then
            -- Not using 'showAnimalDialog' so I can set the last selected husbandry
            local dialog = g_gui:showDialog("AnimalDialog")

            if dialog ~= nil then
                dialog.target:setTitle(g_i18n:getText("category_animalpens"))
                dialog.target:setText("")

                dialog.target:setHusbandries(husbandries)

                -- Apply the last selection to make it easier for repeat usage
                if lastTargetHusbandryIndex > 1 and dialog.target.husbandriesElement ~= nil then
                    dialog.target.husbandriesElement:setState(lastTargetHusbandryIndex, true)
                end

                dialog.target:setCallback(function(targetHusbandry)
                    if targetHusbandry ~= nil then
                        self.lastTargetHusbandry = targetHusbandry

                        -- If there is no animals to move display message but still remember last selected husbandry
                        if sourceHusbandry:getNumOfAnimals() == 0 and targetHusbandry:getNumOfAnimals() == 0 then
                            local text = g_i18n:getText("ui_messageNoAnimalsToMove", AnimalLoadingTriggerMoveActivatable.MOD_NAME)

                            g_gui:showInfoDialog({
                                text = text:format(sourceHusbandry:getName(), targetHusbandry:getName())
                            })

                            return
                        end

                        local infoText = g_i18n:getText("ui_moveAnimalsInfo", AnimalLoadingTriggerMoveActivatable.MOD_NAME)
                        local controller = AnimalScreenMoveToHusbandry.new(sourceHusbandry, targetHusbandry, infoText)

                        controller:init()

                        g_animalScreen:setController(controller)
                        g_gui:showGui("AnimalScreen")
                    end
                end, nil, nil)
            end
        else
            g_gui:showInfoDialog({
                text = g_i18n:getText("ui_messageNoMatchingHusbandryType", AnimalLoadingTriggerMoveActivatable.MOD_NAME)
            })
        end
    end
end
